//+------------------------------------------------------------------+
//|                                                      ay-vwap.mq4 |
//|                      Copyright  2011, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Copyright  2011, MetaQuotes Software Corp."
#property link      "http://www.metaquotes.net"

#property indicator_chart_window
#property indicator_buffers 7

//--- input parameters
extern string    VWAPTfInfo   = "use H4, D1, W1, MN1";
extern string    VWAPTf       = "W1";
extern int       NumVWAPTf    = 2;
extern string    DayStartTime = "00:00";
extern color     ClrVWAP      = CadetBlue;
extern color     ClrSD        = DarkSlateGray;
extern int       VWAPWidth    = 2;
extern int       StyleSD      = STYLE_DOT;

//--- buffers
double BuffUpperSD3[];
double BuffUpperSD2[];
double BuffUpperSD1[];
double BuffVWAP[];
double BuffLowerSD1[];
double BuffLowerSD2[];
double BuffLowerSD3[];


int giVWAPTf;

double gdVWMA[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators
   int i;
   giVWAPTf = getTfByName(VWAPTf);
   
   SetIndexStyle(3, DRAW_LINE, STYLE_SOLID, VWAPWidth, ClrVWAP);
   
   for (i=0; i<7; i++)
   {
      
      SetIndexEmptyValue(i, EMPTY_VALUE);
      if (i==3) continue;
      SetIndexStyle(i, DRAW_LINE, StyleSD, EMPTY, ClrSD);      
   }
   
   SetIndexBuffer(0,BuffUpperSD3);
   SetIndexBuffer(1,BuffUpperSD2);   
   SetIndexBuffer(2,BuffUpperSD1);   
   SetIndexBuffer(3,BuffVWAP);
   SetIndexBuffer(4,BuffLowerSD1);
   SetIndexBuffer(5,BuffLowerSD2);
   SetIndexBuffer(6,BuffLowerSD3);
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if ( Period() >= giVWAPTf ) return(0);
   
   int      counted_bars = IndicatorCounted();
   int      i, j, iBarVWAPTf, iStartBar, iLimit, iNumBars;
   double   dBarVol, dSumVol, dPivotVol, dVWMA, dSD;
   datetime dtVWAPTf0, dtVWAPTf;
   
//---- check for possible errors
   if(counted_bars<0) return(-1);
   if(Period() >= giVWAPTf) return(-1);
//---- last counted bar will be recounted
   if(counted_bars>0) counted_bars--;
   
   iLimit     = Bars-counted_bars;      
   iBarVWAPTf = NumVWAPTf-1;
      
   if (giVWAPTf == PERIOD_D1)
   {      
      //--daystarttime is in future
      if ( createDt(Time[0], DayStartTime) > Time[0] ) 
      {
         iBarVWAPTf++;
         dtVWAPTf = iTime(NULL, giVWAPTf, iBarVWAPTf);
         dtVWAPTf = createDt(dtVWAPTf, DayStartTime);
      }
      else dtVWAPTf  = createDt(dtVWAPTf, DayStartTime);
   }
   else dtVWAPTf   = iTime(NULL, giVWAPTf, iBarVWAPTf);
   
   iStartBar = iBarShift(NULL, 0, dtVWAPTf);      
   if (Time[iStartBar] < dtVWAPTf) iStartBar--;
   
   iLimit = MathMin(iLimit, iStartBar);         

   for (i=iLimit; i>=0; i--)
   {
      
      if (giVWAPTf == PERIOD_D1)
      {
         dtVWAPTf  = createDt(Time[i], DayStartTime);
         if (dtVWAPTf > Time[i]) dtVWAPTf -= PERIOD_D1*60;
      }
      else
      {
         iBarVWAPTf = iBarShift(NULL, giVWAPTf, Time[i]);
         dtVWAPTf   = iTime(NULL, giVWAPTf, iBarVWAPTf);
      }      
      
      iStartBar  = iBarShift(NULL, 0, dtVWAPTf);      
      if (Time[iStartBar] < dtVWAPTf) iStartBar--;                          
             
      iNumBars = (iStartBar-i)+1;
      dVWMA    = getVWMA(iNumBars, i);            
                             
      if (iNumBars<=1) continue;
      
      dSD             = getStdDev(dVWMA, iNumBars, i); 
      
      BuffVWAP[i]     = dVWMA;           
      
      if (ClrSD == CLR_NONE) continue;
             
      BuffUpperSD1[i] = BuffVWAP[i] + (1.0 * dSD);
      BuffUpperSD2[i] = BuffVWAP[i] + (2.0 * dSD);
      BuffUpperSD3[i] = BuffVWAP[i] + (3.0 * dSD);
      BuffLowerSD1[i] = BuffVWAP[i] - (1.0 * dSD);
      BuffLowerSD2[i] = BuffVWAP[i] - (2.0 * dSD);
      BuffLowerSD3[i] = BuffVWAP[i] - (3.0 * dSD);      

   }         
   
   return(0);
   
}
  
datetime createDt(datetime date, string time)
{
   return ( StrToTime(TimeToStr(date, TIME_DATE) + " " + time) );
}

double getVWMA(int period, int shift)
{
   double Sum    = 0;
   double Weight = 0;

   for(int i = shift; i < (shift + period); i++)
   { 
      Weight+= Volume[i];
      Sum += getAppliedPrice(PRICE_TYPICAL, i)*Volume[i];
   }
   
   if(Weight>0) double vwma = Sum/Weight;
   else vwma = 0; 
   
   return(vwma);   

}

double getStdDev(double mean, int period, int shift)
{
   double sum, val, sumvol;  

   for(int i = shift; i < (shift+period); i++)
   {     
      sumvol += Volume[i];     
      val = getAppliedPrice(PRICE_TYPICAL, i) - mean;
      val *= val;

      sum += (Volume[i]) * val;
      //else sum += val;
   }  
   if (sumvol>0)
      return ( MathSqrt(sum/sumvol) );
   else return(0.0);
}
  
/**
 *getTfByName
 */
int getTfByName(string tfname)
{
   int p;
   
        if (tfname == "MN1") p = PERIOD_MN1;
   else if (tfname == "W1" ) p = PERIOD_W1;
   else if (tfname == "D1" ) p = PERIOD_D1;
   else if (tfname == "H4" ) p = PERIOD_H4;
   else if (tfname == "H1" ) p = PERIOD_H1;
   else if (tfname == "M30") p = PERIOD_M30;
   else if (tfname == "M15") p = PERIOD_M15;
   else if (tfname == "M5" ) p = PERIOD_M5;
   else if (tfname == "M1" ) p = PERIOD_M1;
   else p = Period();  
  
   return(p);  

} 
//--copy from market statistic 7
double getAppliedPrice(int nAppliedPrice, int nIndex)
{
   double dPrice;
   //----
   switch(nAppliedPrice)
   {
      case 0:  dPrice=Close[nIndex];                                  break;
      case 1:  dPrice=Open[nIndex];                                   break;
      case 2:  dPrice=High[nIndex];                                   break;
      case 3:  dPrice=Low[nIndex];                                    break;
      case 4:  dPrice=(High[nIndex]+Low[nIndex])/2.0;                 break;
      case 5:  dPrice=(High[nIndex]+Low[nIndex]+Close[nIndex])/3.0;   break;
      case 6:  dPrice=(High[nIndex]+Low[nIndex]+2*Close[nIndex])/4.0; break;
      default: dPrice=0.0;
   }
   //----
   return(dPrice);
}
//+------------------------------------------------------------------+